<?php
abstract class DefaultTransformer 
{
    /**
     * @param $rules
     * @return mixed
     */
    public function transform($rules)
    {
        foreach($rules as &$rule)
            $rule = $this->transformRule($rule);
        return $rules;
    }

    /**
     * @param $rule
     * @return stdClass
     */
    public function  transformRule($rule)
    {
        $newRule = new stdClass();
        $newRule->selectors = $this->transformSelectors($rule->selectors);
        $newRule->properties = $this->transformProperties($rule->properties);
        $newRule->parent = $rule->parent;
        return $newRule;
    }

    /**
     * @param $selectors
     * @return mixed
     */
    protected function transformSelectors($selectors)        
    {
        return $selectors;
    }

    /**
     * @param $properties
     * @return mixed
     */
    protected function transformProperties($properties)        
    {
        return $properties;
    }
}

class PrintCssTransformer  extends DefaultTransformer
{
    /**
     * @param $selectors
     * @return array|mixed
     */
    public function  transformSelectors($selectors)
    {
        $excludedSelectors = array('slider', 'arrow',
          'loading', 'close', 'cw', 'ccw', 'preview-cms-logo',
          'lightbox', 'reset');
        $re = '/(' . implode('|', $excludedSelectors) . ')/';

        $result = array();
        foreach($selectors as $value) {
          $selectorsValidChrs = '-?[_a-zA-Z]+[_a-zA-Z0-9-]*';
          $collapsed = '/\.' . $selectorsValidChrs . 'postcontent' . $selectorsValidChrs . '/';
          $value = trim(preg_replace($collapsed, '', $value));
          if ('' !== $value && !preg_match($re, $value))
              array_push($result, $value);
        }
        return $result;
    }

    /**
     * @param $properties
     * @return array|mixed
     */
    public function transformProperties($properties)
    {
        $result = array();
        foreach($properties as $property => $data){
            if (!preg_match('/(border-|color|background-)/', $property)){
                $result[$property] = $data;
            }
        }
        return $result;
    }
}

class EditorCssTransformer  extends DefaultTransformer
{
    /**
     * @param $selectors
     * @return array|mixed
     */
    public function  transformSelectors($selectors)
    {
        $excludedSelectors = array('slider', 'arrow',
            'loading', 'close', 'cw', 'ccw', 'preview-cms-logo',
            'lightbox', 'reset');
        $re = '/(' . implode('|', $excludedSelectors) . ')/';

        $result = array();
        foreach($selectors as $value) {
            $selectorsValidChrs = '-?[_a-zA-Z]+[_a-zA-Z0-9-]*';
            $collapsed = '/(\.' . $selectorsValidChrs . 'postcontent' . $selectorsValidChrs . ')/';
            $value = preg_replace('/^\.' . $selectorsValidChrs . 'postcontent' . $selectorsValidChrs . ' (\w*)/', 'body $1', $value);
            $value = trim(preg_replace($collapsed, '', $value));
            $value = '' !== $value ? $value : 'body';
            if(!preg_match($re, $value))
                array_push($result, $value);
        }
        return $result;
    }

    /**
     * @param $rule
     * @return stdClass
     */
    public function transformRule($rule)
    {
         $selectors = $this->transformSelectors($rule->selectors);
         $newRule = new stdClass();
         $newRule->selectors = $selectors;
         $newRule->properties = $this->transformProperties($rule->properties);
         $newRule->parent = $rule->parent;
         if (false !== in_array('body', $selectors)){
                $properties = array();
                foreach($rule->properties as $property => $data){
                     switch ($property) {
                        case 'overflow':
                        case 'position':
                        case 'width':
                        case 'min-width':
                            break;
                        default:
                            $properties[$property] = $data;
                            break;
                    }
                }
                $newRule->properties = $properties; 
         }
         return $newRule;           
    }

}